from abc import ABC
from abc import abstractmethod


class LoadConfig(ABC):

    @abstractmethod
    def __init__(self, root_path: str = None, path_file: str = None):
        raise NotImplemented

    @property
    @abstractmethod
    def path_properties(self) -> str:
        raise NotImplemented


class Properties(ABC):

    @abstractmethod
    def __init__(self, root_path: str = None, path_file: str = None):
        raise NotImplemented

    @property
    @abstractmethod
    def __dict__(self) -> dict:
        raise NotImplemented




from properties_loader.inmutables import _MsgError


class PropertiesNotFoundError(Exception):
    def __init__(self, root_path: str, message: str = str(_MsgError.ERROR_TO_LOAD_PROPERTIES_FILE)):
        super(PropertiesNotFoundError, self).__init__(f'{message} -> {root_path}')



import os
from backports.strenum import StrEnum

class _EnvVars(StrEnum):
    ROOT_PATH = 'ROOT_PATH'
    CONFIG_FILE = 'CONFIG_FILE'

class _PriorityProperties(StrEnum):
    PARAMS = '1'
    ENVS_VARS = '2'
    DEFAULT = '3'


class _DfValues(StrEnum):
    DF_ROOT_PATH = str(os.getcwd())
    DF_PATH_FILE = 'properties.ini'


class _MsgError(StrEnum):
    ERROR_TO_LOAD_PROPERTIES_FILE = 'Error al cargar el archivo de properties'
    ROOT_PATH_VARIABLE_NOT_DEFINED = 'ROOT_PATH variable not defined'


import os
from os.path import exists
from configparser import ConfigParser
from properties_loader.inmutables import _EnvVars
from properties_loader.inmutables import _DfValues
from properties_loader.interfaces import LoadConfig, Properties
from properties_loader.inmutables import _PriorityProperties
from properties_loader.exceptions import PropertiesNotFoundError


class LoadConfigImpl(LoadConfig):

    _priority: int = 0
    _root_path: str
    _path_file: str

    def __init__(self, root_path: str = None, path_file: str = None):
        self._root_path = root_path
        self._path_file = path_file
        self.__load_parameters()
        if self._priority == 0:
            self.__load_env_vars()
        if self._priority == 0:
            self.__load_default()
        if self._priority == 0:
            raise PropertiesNotFoundError(path_file)

    def __load_parameters(self):
        root_path = self._root_path
        path_file = self._path_file
        if root_path and path_file and self.__check_properties():
            self._priority = int(_PriorityProperties.PARAMS)

    def __load_env_vars(self):
        self._root_path = os.getenv(_EnvVars.ROOT_PATH)
        self._path_file = os.getenv(_EnvVars.CONFIG_FILE)
        root_path = self._root_path
        path_file = self._path_file
        if root_path and path_file and self.__check_properties():
            self._priority = int(_PriorityProperties.ENVS_VARS)

    def __load_default(self):
        self._root_path = _DfValues.DF_ROOT_PATH
        self._path_file = _DfValues.DF_PATH_FILE
        if self.__check_properties():
            self._priority = int(_PriorityProperties.ENVS_VARS)
        else:
            path_file: str = f"{self._root_path}/{self._path_file}"
            raise PropertiesNotFoundError(path_file)

    def __check_properties(self):
        return exists(f'{self._root_path}/{self._path_file}')

    @property
    def path_properties(self) -> str:
        return f'{self._root_path}/{self._path_file}'


class PropertiesImpl(LoadConfigImpl, Properties):

    _root_path: str
    _path_file: str

    def __init__(self, root_path: str = None, path_file: str = None):
        super(PropertiesImpl, self).__init__(root_path=root_path, path_file=path_file)

    def __load_configparser(self):
        config_parser = ConfigParser()
        path_file: str = f"{self._root_path}/{self._path_file}"
        try:
            config_parser.read(path_file)
        except Exception:
            from properties_loader.exceptions import PropertiesNotFoundError
            raise PropertiesNotFoundError(path_file)
        else:
            return config_parser

    @property
    def __dict__(self) -> dict:
        properties_to_json: dict = {}
        config_parser: ConfigParser = self.__load_configparser()
        for section in config_parser.sections():
            keys_values_json = {}
            for key in config_parser.options(section):
                keys_values_json[key] = config_parser[section][key]
            properties_to_json[section] = keys_values_json
        return properties_to_json




class PropertiesClassLoader(PropertiesImpl):
    __groups: list

    def __init__(self, root_path: str = None, path_file: str = None, groups: list = None):
        super(PropertiesClassLoader, self).__init__(root_path=root_path, path_file=path_file)
        self.__load_parameters(groups)
        self.__efective_properties = self.__load_efective_properties()
        self.__load_properties()

    def __load_parameters(self, groups: list):
        self.__groups = self.__load_groups(groups)

    def __load_properties(self):
        for group in self.__efective_properties.keys():
            setattr(self, f'_{group}', self.__efective_properties.get(group))

    def __load_efective_properties(self):
        if self.__groups is None:
            return self.__dict__

        json_data: dict = {}
        for group in self.__dict__.keys():
            if group in self.__groups:
                json_data[group] = self.__dict__.get(group)
        return json_data

    def __load_groups(self, groups):
        if groups:
            return groups
        return []

    @property
    def efective_properties(self):
        return self.__efective_properties



import setuptools
import pathlib

HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setuptools.setup(
    name="cleandev_config_loader",
    version="0.3.4",
    author="Daniel Rodriguez Rodriguez",
    author_email="danielrodriguezrodriguez.pks@gmail.com",
    description="Loader for properties.ini make more easy",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/cleansoftware/libs/public/config-loader",
    project_urls={
        "Bug Tracker": "https://gitlab.com/cleansoftware/libs/public/config-loader/-/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=["properties_loader"],
    install_requires=[
        'backports.strenum==1.1.1'
    ],
    python_requires=">=3.9",
)
